#---
# Excerpted from "The ThoughtWorks Anthology 2",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/twa2 for more book information.
#---
class BlogPost
  attr_reader :title, :updated_at, :content
  def initialize(params)
    @title = params[:title]
    @updated_at = params[:updated_at]
    @content = params[:content]
  end
  def current_month?
    today = Time.now
    @updated_at.year == today.year && @updated_at.month == today.month
  end
  def truncated_content(word_limit = 100)
    # ...
  end
  def eql?(other)
    return true if equal?(other)
    @title == other.title && @updated_at == other.updated_at && 
      @content == other.content
  end
  alias_method :==, :eql?
  def hash
    @title.hash + @updated_at.hash + @content.hash
  end
end

class BlogAuthor
  attr_reader :name, :email
  def initialize(params)
    @name = params[:name]
    @email = params[:email]
  end
  def anonymous?
    @name.empty? && @email.empty?
  end
  
  def display
    return 'Unknown' if anonymous?
    @name || @email
  end
  def eql?(other)
    return true if equal?(other)
    @name == other.name && @email == other.email
  end
  alias_method :==, :eql?
  def hash
    @name.hash + @email.hash
  end
end

now = Time.now
blog_post1 = BlogPost.new(:title => 'Some Title', :updated_at => now, :content => 'some content')
blog_post2 = BlogPost.new(:title => 'Some Title', :updated_at => now, :content => 'some content')
puts blog_post1 == blog_post2 #=> true
puts blog_post1.current_month? #=> true
puts blog_post1.title #=> Some Title

blog_author1 = BlogAuthor.new(:name => 'Some Name', :email => 'test@test.com')
blog_author2 = BlogAuthor.new(:name => 'Some Name', :email => 'test@test.com')
puts blog_author1 == blog_author2 #=> true
puts blog_author1.anonymous? #=> false
puts blog_author1.name #=> Some Name
